__author__ = "Managed by Q, Inc."
__author_email__ = "open-source@managedbyq.com"
__description__ = "MBQ PubSub"
__license__ = "Apache 2.0"
__title__ = "mbq.pubsub"
__url__ = "https://github.com/managedbyq/mbq.pubsub"
__version__ = "0.0.3"
